describe :receive_and_throw do

  it 'should pass when received message throws whatever' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {throw :z}}
        expect(x).to_receive(:y).and_throw
        x.y
      end
      this.assert_equal :__tokyo_passed__, run(:test)
    end
  end

  it 'should pass when received message throws expected symbol' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {throw :z}}
        expect(x).to_receive(:y).and_throw(:z)
        x.y
      end
      this.assert_equal :__tokyo_passed__, run(:test)
    end
  end

  it 'should fail when received message throws nothing' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {}}
        expect(x).to_receive(:y).and_throw
        x.y
      end
      this.assert_match /Expected a symbol to be thrown/, run(:test).reason*' '
    end
  end

  it 'should fail when received message throws wrong symbol' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {throw :z}}
        expect(x).to_receive(:y).and_throw(:a)
        x.y
      end
      this.assert_match /Expected :a to be thrown/, run(:test).reason*' '
    end
  end

  it 'should pass when given block validates thrown symbol' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {throw :z}}
        expect(x).to_receive(:y).and_throw {|s| s == :z}
        x.y
      end
      this.assert_equal :__tokyo_passed__, run(:test)
    end
  end

  it 'should fail when block does not validate thrown symbol' do
    this = self
    spec rand do
      test :test do
        x = Class.new {define_singleton_method(:y) {throw :z}}
        expect(x).to_receive(:y).and_throw {false}
        x.y
      end
      this.assert_match /Looks like wrong or no symbol thrown/, run(:test).reason*' '
    end
  end
end
